<?php

namespace Drupal\library_manager\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\library_manager\LibraryDiscoveryInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds the form to create a library definition from existing library.
 *
 * @property \Drupal\library_manager\LibraryDefinitionInterface $entity
 */
class LibraryDefinitionBuildForm extends EntityForm {

  /**
   * The library discovery service.
   *
   * @var \Drupal\library_manager\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * Constructs the form object.
   *
   * @param \Drupal\library_manager\LibraryDiscoveryInterface $library_discovery
   *   The discovery service.
   */
  public function __construct(LibraryDiscoveryInterface $library_discovery) {
    $this->libraryDiscovery = $library_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('library_manager.library_discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $extension = $this->getRequest()->get('extension');
    $library = $this->getRequest()->get('library');
    if (!(is_string($extension) && is_string($library) && $this->libraryDiscovery->getLibraryByName($extension, urldecode($library)))) {
      throw new NotFoundHttpException();
    }

    $form = parent::form($form, $form_state);

    $form['id'] = [
      '#type' => 'machine_name',
      '#title' => t('Library definition ID'),
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\library_manager\Entity\LibraryDefinition::load',
      ],
    ];

    $url = Url::fromRoute(
      'library_manager.library_canonical',
      ['extension' => $extension, 'library' => $library]
    );

    $form['source'] = [
      '#type' => 'item',
      '#title' => t('Source library'),
      '#markup' => Link::fromTextAndUrl($extension . '/' . $library, $url)->toString(),
    ];

    $form['extension'] = [
      '#type' => 'value',
      '#value' => $extension,
    ];

    $form['library'] = [
      '#type' => 'value',
      '#value' => $library,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->entity->set('id', $values['id']);
    $this->updateLibraryDefinition($values['extension'], $values['library']);
    $form_state->setRedirectUrl($this->entity->toUrl('edit-form'));
    return $this->entity->save();
  }

  /**
   * Updates library definition according to library info.
   *
   * @param string $extension
   *   The name of the extension that registered a library.
   * @param string $library
   *   The name of a registered library to retrieve information.
   */
  protected function updateLibraryDefinition($extension, $library) {
    $library_info = $this->libraryDiscovery->exportLibraryByName($extension, $library);
    $extension_path = $this->libraryDiscovery->getExtensionPath($extension);

    if (isset($library_info['version'])) {
      $version = $this->libraryDiscovery->processLibraryVersion($library_info['version']);
      $this->entity->set('version', $version);
    }
    if (isset($library_info['license'])) {
      $this->entity->set('license', $library_info['license']);
    }

    // Define JS files.
    if (isset($library_info['js'])) {
      $js_files = [];
      $file_id = 1;
      foreach ($library_info['js'] as $file_name => $file) {
        $js_files[$file_id]['file_name'] = basename($file_name);
        $js_files[$file_id]['preprocess'] = !empty($file['preprocess']);
        $js_files[$file_id]['minified'] = !empty($file['minified']);
        $js_files[$file_id]['external'] = !empty($file['external']);
        $js_files[$file_id]['weight'] = isset($file['weight']) ? $file['weight'] : 0;
        if ($js_files[$file_id]['external']) {
          $js_files[$file_id]['code'] = '';
          $js_files[$file_id]['url'] = $file[$file_name];
          $js_files[$file_id]['external'] = TRUE;
        }
        else {
          $file_path = $extension_path . $file_name;
          $js_files[$file_id]['code'] = file_get_contents($file_path);
          $js_files[$file_id]['url'] = '';
        }
        $file_id++;
      }
      $this->entity->set('js', $js_files);
    }

    // Define CSS files.
    if (isset($library_info['css'])) {
      $css_files = [];
      $file_id = 1;
      foreach ($library_info['css'] as $group => $group_files) {
        foreach ($group_files as $file_name => $file) {
          $css_files[$file_id]['file_name'] = basename($file_name);
          $css_files[$file_id]['group'] = $group;
          $css_files[$file_id]['preprocess'] = !empty($file['preprocess']);
          $css_files[$file_id]['minified'] = !empty($file['minified']);
          $css_files[$file_id]['external'] = !empty($file['external']);
          $css_files[$file_id]['weight'] = isset($file['weight']) ? $file['weight'] : 0;
          if ($css_files[$file_id]['external']) {
            $css_files[$file_id]['code'] = '';
            $css_files[$file_id]['url'] = $file[$file_name];
            $css_files[$file_id]['external'] = TRUE;
          }
          else {
            $file_path = $extension_path . $file_name;
            $css_files[$file_id]['code'] = file_get_contents($file_path);
            $css_files[$file_id]['url'] = '';
          }
          $file_id++;
        }
      }

      $this->entity->set('css', $css_files);
    }

    if (isset($library_info['dependencies'])) {
      $this->entity->set('library_dependencies', $library_info['dependencies']);
    }
  }

}
